package Kirjalista;

import java.util.NoSuchElementException;

/**
 * Monikkoluokka kirjoille.
 * - Sisältää taulukon kirjoista
 * - Lisää ja poistaa kirjoja
 * - Lukee ja kirjoittaa tiedostoon (sitten joskus)
 * - Etsii ja lajittelee (sitten joskus)
 * <pre name="test">
 * #import java.util.NoSuchElementException;
 * #THROWS NoSuchElementException
 * Kirjat kirjat = new Kirjat();
 * 
 * Kirja k1 = new Kirja();
 * Kirja k2 = new Kirja();
 * Kirja k3 = new Kirja();
 * k1.parse("Hobitti1 |J.R.R. Tolkien||336");
 * k2.parse("Hobitti2 |J.R.R. Tolkien||336");
 * k3.parse("Hobitti3 |J.R.R. Tolkien||336");
 * 
 * 
 * 
 * kirjat.lisaa(k1);
 * kirjat.lisaa(k2);
 * kirjat.lisaa(k3);
 * 
 * 
 * kirjat.hae(k1.getId()).getNimi() === "Hobitti1";
 * kirjat.hae(k3.getId()).getNimi() === "Hobitti3";
 * kirjat.hae(0); #THROWS NoSuchElementException
 * kirjat.hae(5); #THROWS NoSuchElementException
 * 
 * int k2Id = k2.getId();
 * kirjat.poista(k2Id);
 * kirjat.poista(0); #THROWS NoSuchElementException
 * kirjat.hae(k2Id); #THROWS NoSuchElementException
 * 
 * </pre>
 */
public class Kirjat {
    private int maxLkm = 8;
    private int lkm = 0;
    private Kirja[] alkiot = new Kirja[maxLkm];
    
    
    /**
     * @param kirja Lisättävä kirja
     * <pre name="test">
     * Kirja kirja = new Kirja();
     * Kirjat kirjat = new Kirjat();
     * for (int i = 0; i < 8; i++) {
     *     kirjat.lisaa(kirja);
     *     kirjat.getLkm() === i + 1;
     * }
     * kirjat.getLkm() === 8;
	 * 
	 * 
	 * </pre>
     */
    public void lisaa(Kirja kirja) {
        if (this.lkm >= this.alkiot.length) {
            Kirja[] uusi = new Kirja[this.lkm * 2];
            for (int i = 0; i < this.lkm; i++) {
                uusi[i] = this.alkiot[i];
            }
            this.alkiot = uusi;
        }
        this.alkiot[this.lkm] = kirja;
        kirja.rekisteroi();
        this.lkm++;
    }
    
    
    /**
     * Poistaa kirjan annetulla id:llä
     * @param id Poistettavan kirjan id
     * @throws NoSuchElementException Jos id:tä vastaavaa kirjaa ei löydy
     */
    public void poista(int id) throws NoSuchElementException {
        int luku, kirj;
        for (luku = 0, kirj = 0; luku < this.lkm; luku++) {
            if (this.alkiot[luku].getId() != id)  {
                this.alkiot[kirj++] = this.alkiot[luku];
            }
        }
        this.lkm = kirj;
        if (luku == kirj) throw new NoSuchElementException("Kirjaa id:llä " + id + " ei löydy.");
    }

    /**
     * Hakee kirjan id:n perusteella
     * @param id Halutun kirjan id
     * @return Viite kirjaan halutulla id:llä
     * @throws NoSuchElementException Jos Id:tä vastaavaa kirjaa ei löydy
     */
    public Kirja hae(int id) throws NoSuchElementException {
        
        for (int i = 0; i < this.lkm; i++) {
            if (alkiot[i].getId() == id) return alkiot[i];
        }
        
        throw new NoSuchElementException("Kirjaa id:llä " + id + " ei löytynyt.");
    }

    
    /**
     * Hakee kirjan indeksin perusteella
     * @param ind Indeksipaikka, josta kirja haetaan
     * @return Kirja annetussa indeksipaikassa
     */
    public Kirja haeIndeksilla(int ind) {
        if (ind < 0 || ind >= this.lkm) throw new NullPointerException("Laiton indeksi");
        return alkiot[ind];
    }
    
    
    /**
     * Palauttaa kirjojen lukumäärän
     * @return kirjojen lukumäärä
     */
    public int getLkm() {
        return this.lkm;
    }

}
